function  [A_cor,B_cor,C_cor]  =    amb_correct_1f_beta(beta,A_p,B_p,C_p,C)
%-------------------------------------------------------------------------%
%         [A_cor,B_cor,C_cor]  =    amb_correct_1f_beta(beta,A_p,B_p,C_p,C)
% Correction of the permutation ambiguity of the CP model for positive 
% models. Correction is done by minimizing a beta divergence.
%
% The inputs are:
%
% - beta        : Beta divergence parameter (beta>0);
% - A_p,B_p,C_p : estimated factors without correction;
% - C           : estimated coupled factor of the first tensor.
%
% The outputs are:
% 
% - A_cor,...    : factors with corrected sign and permutations.
%
% List of updates                 -     23/02/2015  -     Rodrigo Cabral
%                                       Creation of the file
%                                 -     11/02/2016  -     R. C. Farias
%                                       Modification for beta divergence
%-------------------------------------------------------------------------%

%-------------------------Permutation correction--------------------------%
% Number of components
R   =     size(A_p,2);

% Repeated parameters for evaluation of the true error
p         =     perms(1:R)';
X_rep     =     C(:);
X_rep     =     repmat(X_rep,1,length(p));
% Possible permutations
X_hat     =     C_p;
X_hat_perm      =     X_hat(:,p(:)');
X_hat_perm      =     reshape(X_hat_perm,[size(X_hat,1)*R,length(p)]);

% Evaluation of the beta divergences and optimal permutations
beta_div        =     zeros(1,length(p));
for       i     =     1:length(p)
     beta_div(i)=     beta_div_eval(X_hat_perm(:,i),X_rep(:,i),beta);
end
[~,I_hat]       =     min(beta_div);

% Permutation of the columns
A_cor     =     A_p(:,p(:,I_hat)');
B_cor     =     B_p(:,p(:,I_hat)');
C_cor     =     C_p(:,p(:,I_hat)');
%-------------------------------------------------------------------------%
end